#!/usr/bin/perl

# CGI stuff, useful for param()
use CGI qw/:standard/;
use strict;

# Needed header printage for CGI scripts
print "Content-type: text/html\n\n";

my $request_type;

# Location of the nagiostat binary
my $nagiostat = '/usr/bin/nagiostats';

# Does the nagiostat binary even exist?
if ( ! -e $nagiostat) {
	print "Specified Nagios binary and/or configuration files does not exist!\n";
	exit;
}

# Get nagios version
my $nagiosver = `$nagiostat -d NAGIOSVERSION -m`;

# Extract major version (first number of $nagiosver)
my ($nagiosmajorver) = ($nagiosver =~ m/^([0-9])/g);

# Do we have a param?
if(param()) {
	$request_type = param('type');
} 

# Param didn't exist or was empty, oh dear!
if($request_type eq "") {
	print "No type parameter given, add '?type=latency' to the URL, for example!\n";
	exit;
}

# Associate graph tyes with fields from nagiostat
my %graphs = ( 	'latency' 	=> ['AVGACTSVCLAT', 'AVGACTSVCEXT','AVGACTHSTLAT','AVGACTHSTEXT'],
		'servicechecks'	=> ['NUMSERVICES','NUMSVCOK','NUMSVCWARN','NUMSVCUNKN','NUMSVCCRIT','NUMSVCPROB','NUMSVCDOWNTIME'],
		'hostchecks'	=> ['NUMHOSTS','NUMHSTUP','NUMHSTDOWN','NUMHSTUNR','NUMHSTPROB','NUMHSTDOWNTIME']
 );

# Silly nagios devs thought it'd be a good idea to rename some fields, you know, for compatbility sake *ahem*
if($nagiosmajorver == 3) {
	$graphs{'checkstats'} = ['NUMHSTACTCHK5M','NUMSVCACTCHK5M','NUMHSTPSVCHK5M','NUMSVCPSVCHK5M'];
} elsif($nagiosmajorver == 2) {
	$graphs{'checkstats'} = ['NUMACTHSTCHK5M','NUMACTSVCCHK5M','NUMPSVHSTCHK5M','NUMPSVSVCCHK5M'];
} else {
	print "Nagios version '$nagiosver' not supported..\n";
	exit;
}
# The requested type does not exist in the hash
if (! defined($graphs{$request_type})) {
  print "Type $request_type is not a valid request type!\n";
  exit;
}

# construct parameters for command line
my $params;
foreach my $i (@{$graphs{$request_type}}) {
  if (! defined($params)) {
    $params = $i;
  }
  else {
    $params = $params . ",$i";
  }
}

# execute command and collect results
open(RESULTS, "$nagiostat -d $params -m|");
  
my $index=0;
while (<RESULTS>) {
	chomp;
	my $tmp = "$graphs{$request_type}[$index]:$_ ";
	# Make nagiostats from nagios 2 and 3 compatible
	$tmp =~ s/ACTHSTCHK/HSTACTCHK/;
	$tmp =~ s/ACTSVCCHK/SVCACTCHK/;
	$tmp =~ s/PSVHSTCHK/HSTPSVCHK/;
	$tmp =~ s/PSVSVCCHK/SVCPSVCHK/;
	print "$tmp";
	$index++;
}
print "\n";

close RESULTS;
